package cache

import (
	"context"
	"encoding/json"
	model "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/base"
	"go.mongodb.org/mongo-driver/bson"
	primitive "go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"log"
	"sync"
)

type ChangeStream[T any] interface {
	DocumentEqual(a, b T) bool
	Filter(data ChangeStreamData) bson.M
}

type Cache[T any] interface {
	GetCacheDocuments() []T
	StartCache(ctx context.Context) error
	GetCacheOne(hitFunc func(document T) bool) (T, bool)
	GetCacheMany(hitFunc func(document T) bool) []T
}

type Base[T any] struct {
	baseModel    model.Model[T]
	changeStream ChangeStream[T]
	documents    []T
	rwMutex      *sync.RWMutex
}

func NewBase[T any](baseModel model.Model[T], changeStream ChangeStream[T]) *Base[T] {
	return &Base[T]{baseModel: baseModel, changeStream: changeStream, rwMutex: &sync.RWMutex{}}
}

func (receiver *Base[T]) StartCache(ctx context.Context) error {
	documents, _, err := receiver.baseModel.Find(ctx, bson.M{})
	if err != nil {
		return err
	}
	receiver.setAllDocuments(documents)
	go func() {
		for {
			changeStream, err := receiver.baseModel.Watch(ctx, mongo.Pipeline{})
			if err != nil {
				log.Println(err)
				continue
			}
			for changeStream.Next(context.TODO()) {
				var changeStreamData ChangeStreamData
				changeBody := changeStream.Current.String()
				err = json.Unmarshal([]byte(changeBody), &changeStreamData)
				if err != nil {
					log.Println(err)
					continue
				}
				filter := receiver.changeStream.Filter(changeStreamData)
				document, err := receiver.baseModel.FindOne(ctx, filter)
				if err != nil {
					log.Println(err)
					continue
				}
				documents := receiver.GetCacheDocuments()
				for i := range documents {
					if receiver.changeStream.DocumentEqual(documents[i], document) {
						documents[i] = document
					}
				}
				receiver.setAllDocuments(documents)
			}
			err = changeStream.Close(context.TODO())
			if err != nil {
				log.Println(err)
				continue
			}
		}
	}()
	return nil
}

func (receiver *Base[T]) GetCacheDocuments() []T {
	receiver.rwMutex.RLock()
	defer receiver.rwMutex.RUnlock()
	tmp := make([]T, len(receiver.documents))
	copy(tmp, receiver.documents)
	return tmp
}

func (receiver *Base[T]) setAllDocuments(documents []T) {
	receiver.rwMutex.Lock()
	defer receiver.rwMutex.Unlock()
	receiver.documents = documents
}

func (receiver *Base[T]) Filter(data ChangeStreamData) bson.M {
	id, err := primitive.ObjectIDFromHex(data.DocumentKey.Id.Oid)
	if err != nil {
		return nil
	}
	return bson.M{"_id": id}
}

func (receiver *Base[T]) GetCacheOne(hitFunc func(document T) bool) (T, bool) {
	for _, d := range receiver.GetCacheDocuments() {
		if hitFunc(d) {
			return d, true
		}
	}
	return *new(T), false
}

func (receiver *Base[T]) GetCacheMany(hitFunc func(document T) bool) []T {
	var documents []T
	for _, d := range receiver.GetCacheDocuments() {
		if hitFunc(d) {
			documents = append(documents, d)
		}
	}
	return documents
}

type ChangeStreamData struct {
	DocumentKey struct {
		Id struct {
			Oid string `json:"$oid"`
		} `json:"_id"`
	} `json:"documentKey"`
}
