package model

import (
	"context"
	"github.com/zeromicro/go-zero/core/stores/mon"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type Model[T any] interface {
	Find(ctx context.Context, filter bson.M, findOptions ...*options.FindOptions) ([]T, int64, error)
	FindOne(ctx context.Context, filter bson.M, findOneOptions ...*options.FindOneOptions) (T, error)
	UpdateOne(ctx context.Context, filter bson.M, update bson.D, updateOptions ...*options.UpdateOptions) (*mongo.UpdateResult, error)
	Traverse(ctx context.Context, filter bson.M, f func(document T) (bool, error), findOptions ...*options.FindOptions) error
	BulkWrite(ctx context.Context, writeModels []mongo.WriteModel, opts ...*options.BulkWriteOptions) (*mongo.BulkWriteResult, error)
	Watch(ctx context.Context, pipeline any, opts ...*options.ChangeStreamOptions) (*mongo.ChangeStream, error)
}

type Base[T any] struct {
	model *mon.Model
}

func NewBase[T any](model *mon.Model) *Base[T] {
	return &Base[T]{model: model}
}

func (receiver *Base[T]) Find(ctx context.Context, filter bson.M, findOptions ...*options.FindOptions) ([]T, int64, error) {
	total, err := receiver.model.CountDocuments(ctx, filter)
	if err != nil {
		return nil, 0, err
	}
	var documents []T
	err = receiver.model.Find(ctx, &documents, filter, findOptions...)
	return documents, total, err
}

func (receiver *Base[T]) FindOne(ctx context.Context, filter bson.M, findOneOptions ...*options.FindOneOptions) (T, error) {
	var document T
	err := receiver.model.FindOne(ctx, &document, filter, findOneOptions...)
	return document, err
}

func (receiver *Base[T]) UpdateOne(ctx context.Context, filter bson.M, update bson.D, updateOptions ...*options.UpdateOptions) (*mongo.UpdateResult, error) {
	return receiver.model.UpdateOne(ctx, filter, update, updateOptions...)
}

func (receiver *Base[T]) Traverse(ctx context.Context, filter bson.M, f func(document T) (bool, error), findOptions ...*options.FindOptions) error {
	cursor, err := receiver.model.Collection.Find(ctx, filter, findOptions...)
	if err != nil {
		return err
	}
	defer cursor.Close(ctx)
	for cursor.Next(ctx) {
		var document T
		err = cursor.Decode(&document)
		if err != nil {
			return err
		}
		next, err := f(document)
		if err != nil {
			return err
		}
		if !next {
			return nil
		}
	}
	return nil
}

func (receiver *Base[T]) BulkWrite(ctx context.Context, writeModels []mongo.WriteModel, opts ...*options.BulkWriteOptions) (*mongo.BulkWriteResult, error) {
	return receiver.model.BulkWrite(ctx, writeModels, opts...)
}

func (receiver *Base[T]) Watch(ctx context.Context, pipeline any, opts ...*options.ChangeStreamOptions) (*mongo.ChangeStream, error) {
	return receiver.model.Watch(ctx, pipeline, opts...)
}
