package model

import (
	"git.chillcy.com/golang/chillcy/pkg/gender"
	"git.chillcy.com/golang/colorway/pkg/types"
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

const Collection = "product"

type Product struct {
	ID           primitive.ObjectID  `bson:"_id,omitempty" json:"id,omitempty"`
	StyleId      string              `bson:"styleId"`     // 可以是分割形式出来的主款号，也可以是品牌官网同款组的 ID
	StyleNumber  string              `bson:"styleNumber"` // 源款号
	Title        ProductTitle        `bson:"title"`
	Description  string              `bson:"description"`
	Season       string              `bson:"season"`
	Brand        ProductBrand        `bson:"brand"`
	Care         string              `bson:"care"`
	MadeIn       string              `bson:"madeIn"`
	HsCode       string              `bson:"hsCode"`
	InfoSelect   string              `bson:"infoSelect"` // 商品信息默认选择哪个商品
	Colors       ProductColors       `bson:"colors"`
	Category     ProductCategory     `bson:"category"`
	Compositions ProductCompositions `bson:"compositions"`
	Fits         []string            `bson:"fits"`
	Genders      gender.Genders      `bson:"genders"`
	Models       []string            `bson:"models"`
	UpdateAt     time.Time           `bson:"updateAt,omitempty" json:"updateAt,omitempty"`
	CreateAt     time.Time           `bson:"createAt,omitempty" json:"createAt,omitempty"`
}

func (receiver Product) FindPlatformStoreSku(platform types.Platform, store types.Store, route types.Route, skuId string) (ProductColorSkuPlatformStoreSku, bool) {
	return receiver.Colors.FindPlatformStoreSku(platform, store, route, skuId)
}

type Products []Product

type ProductComposition struct {
	Part     string `bson:"part"`
	Material string `bson:"material"`
	Value    string `bson:"value"`
}

type ProductCompositions []ProductComposition

type ProductCategory struct {
	Id       string            `bson:"id"`
	Name     string            `bson:"name"`
	Children []ProductCategory `json:"children" bson:"children"`
}

func (receiver ProductCategory) GetIds() []string {
	ids := []string{receiver.Id}
	for _, child := range receiver.Children {
		ids = append(ids, child.GetIds()...)
	}
	return ids
}

// Style 记录原始款号信息
type Style struct {
	Number    string `bson:"number"`
	ColorCode string `bson:"colorCode"`
}

type ProductColor struct {
	ColorId    string           `bson:"colorId"`   // 可以是分割出来的色号，也可以是品牌官网同款组的唯一确定颜色的款号
	ColorCode  string           `bson:"colorCode"` // 源色号
	Id         string           `bson:"id"`
	Name       string           `bson:"name"`
	BasicId    string           `bson:"basicId"`
	BasicName  string           `bson:"basicName"`
	InfoSelect string           `bson:"infoSelect"` // 商品信息默认选择哪个商品
	Skus       ProductColorSkus `bson:"skus"`
	Images     []string         `bson:"images"`
}

type ProductColors []ProductColor

func (receiver ProductColors) FindPlatformStoreSku(platform types.Platform, store types.Store, route types.Route, skuId string) (ProductColorSkuPlatformStoreSku, bool) {
	for _, color := range receiver {
		if productColorSkuPlatformStoreSku, ok := color.Skus.FindPlatformStoreSku(platform, store, route, skuId); ok {
			return productColorSkuPlatformStoreSku, true
		}
	}
	return ProductColorSkuPlatformStoreSku{}, false
}

type ProductColorSku struct {
	Pc3SkuId          string                           `bson:"pc3SkuId"`
	Pc3ProductId      string                           `bson:"pc3ProductId"`
	SourceId          string                           `bson:"sourceId"`
	SiteId            string                           `bson:"siteId"`
	Size              string                           `bson:"size"`
	Stock             int                              `bson:"stock"`
	Price             float64                          `bson:"price"`
	Currency          string                           `bson:"currency"`
	SubjectIds        []string                         `bson:"subjectIds"`
	PlatformStoreSkus ProductColorSkuPlatformStoreSkus `bson:"platformStoreSkus"`
}

type ProductColorSkus []ProductColorSku

func (receiver ProductColorSkus) FindPlatformStoreSku(platform types.Platform, store types.Store, route types.Route, skuId string) (ProductColorSkuPlatformStoreSku, bool) {
	for _, sku := range receiver {
		if sku.Pc3SkuId == skuId {
			return sku.PlatformStoreSkus.FindPlatformStoreSku(platform, store, route)
		}
	}
	return ProductColorSkuPlatformStoreSku{}, false
}

type ProductTitle struct {
	En string `bson:"en"`
	Cn string `bson:"cn"`
}

type ProductBrand struct {
	Id   string `bson:"id"`
	Name string `bson:"name"`
}

type ProductColorSkuPlatformStoreSku struct {
	Platform       types.Platform `bson:"platform"`             // 销售平台
	Store          types.Store    `bson:"store"`                // 销售平台具体店铺
	Route          types.Route    `bson:"route"`                // 线路
	Currency       string         `bson:"currency"`             // 销售平台的币种
	OriginCurrency string         `bson:"originCurrency"`       // 原始币种
	Price          float64        `bson:"price"`                // 最终价格
	FixPrice       *float64       `bson:"fixPrice,omitempty"`   // 固定价格
	FixMargin      *float64       `bson:"fixMargin,omitempty"`  // 固定加价率(影响计价)
	FixedQty       *int           `bson:"fixedQty,omitempty"`   // 虚拟库存
	Rate           float64        `bson:"rate"`                 // OriginCurrency -> Currency 的汇率
	CnyRate        float64        `bson:"cnyRate"`              // Currency -> CNY 的汇率
	Margin         float64        `bson:"margin"`               // 加价率
	SubjectRate    float64        `bson:"subjectRate"`          // 专题系数
	Subject        string         `bson:"subject"`              // 绑定的专题id
	DiscountRate   float64        `bson:"discountRate"`         // 商家系数
	TotalDiscount  float64        `bson:"totalDiscount"`        // 总折扣
	TaxRate        float64        `bson:"taxRate"`              // 税率
	Freight        float64        `bson:"freight"`              // 商家运费
	Express        float64        `bson:"express"`              // 跨境费
	ChangeRate     *float64       `bson:"changeRate,omitempty"` // 价格变动率, 0.1表示涨价10%, -0.1表示降价10%
	ChangeAt       *time.Time     `bson:"changeAt,omitempty"`   // 价格变动时间，方便筛选
	Memos          types.Memos    `bson:"memos,omitempty"`      // 操作备注
}

type ProductColorSkuPlatformStoreSkus []ProductColorSkuPlatformStoreSku

func (receiver ProductColorSkuPlatformStoreSkus) FindPlatformStoreSku(platform types.Platform, store types.Store, route types.Route) (ProductColorSkuPlatformStoreSku, bool) {
	for _, sku := range receiver {
		if sku.Platform == platform && sku.Store == store && sku.Route == route {
			return sku, true
		}
	}
	return ProductColorSkuPlatformStoreSku{}, false
}
