package model

import (
	"git.chillcy.com/golang/chillcy/pkg/currency"
	"git.chillcy.com/golang/colorway/pkg/types"
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

const Collection = "site_platform_price_config"

type SitePlatformPriceConfig struct {
	ID               primitive.ObjectID `bson:"_id,omitempty" json:"id,omitempty"`
	Site             string             `json:"site" bson:"site"`
	Seller           types.Seller       `json:"seller" bson:"seller"` // 销售平台
	Enable           bool               `json:"enable" bson:"enable"` // 是否启用
	RoutePriceConfig `bson:",inline"`   // 路线价格配置
	UpdateAt         time.Time          `bson:"updateAt,omitempty" json:"updateAt,omitempty"`
	CreateAt         time.Time          `bson:"createAt,omitempty" json:"createAt,omitempty"`
}

type RoutePriceConfig struct {
	Route         types.Route       `json:"route" bson:"route"`                 // 路线
	Currency      currency.Currency `json:"currency" bson:"currency"`           // 结算币种
	TaxRate       float64           `json:"taxRate" bson:"taxRate"`             // 税率
	MarginConfig  MarginConfig      `json:"marginConfig" bson:"marginConfig"`   // 加价率配置
	FreightConfig FreightConfig     `json:"freightConfig" bson:"freightConfig"` // 商家运费配置，默认是 0
	ExpressConfig ExpressConfig     `json:"expressConfig" bson:"expressConfig"` // 跨境快递配置，默认是0
}

func (mc MarginConfig) FormatLadders() {
	if mc.MarginStyle == MarginStyleLadder {
		for i := range mc.Ladders {
			if i == len(mc.Ladders)-1 {
				mc.Ladders[i].LastOne = true
			}
		}
	}
}

func (rc RoutePriceConfig) IsValid() bool {
	// 路线价格配置校验-加价率
	if rc.MarginConfig.MarginStyle == MarginStyleFixed && (rc.MarginConfig.Margin <= 0 || rc.MarginConfig.Margin > 1) {
		return false
	}
	if rc.MarginConfig.MarginStyle == MarginStyleLadder && len(rc.MarginConfig.Ladders) <= 0 {
		return false
	}
	if rc.MarginConfig.MarginStyle == MarginStyleLadder {
		if !rc.MarginConfig.IsValidLadders() {
			return false
		}
	}
	// 路线价格配置校验-运费
	if rc.Route == types.RouteCC || rc.Route == types.RouteBC {
		// toC线路必须有税率
		if rc.TaxRate <= 0 {
			return false
		}
	}
	if rc.Route == types.RouteBB && (rc.FreightConfig.FreightStyle == FreightStyleCategory || rc.FreightConfig.FreightStyle == FreightStylePrice) {
		// B2B线路不能按照分类计算运费, 也不能按照页面价格计算运费
		return false
	}
	if rc.Route == types.RouteBB && rc.ExpressConfig.ExpressStyle == ExpressStyleCategory {
		// B2B线路不能按照分类计算快递费
		return false
	}

	return true
}

type ExpressConfig struct {
	Express      float64      `json:"express" bson:"express"`           // 跨境快递费
	ExpressStyle ExpressStyle `json:"expressStyle" bson:"expressStyle"` // 跨境快递费计算方式
}

type ExpressStyle string

const ExpressStyleDefault = "default"   // 默认计算方式
const ExpressStyleCategory = "category" // 按照分类计算方式

type FreightConfig struct {
	Freight             float64           `json:"freight" bson:"freight"`                         // 商家运费
	FreightStyle        FreightStyle      `json:"freightStyle" bson:"freightStyle" `              // 商家运费计算方式
	OverFreight         int               `json:"overFreight" bson:"overFreight"`                 // 计算方式是满免的时候会用到
	OverFreightCurrency currency.Currency `json:"overFreightCurrency" bson:"overFreightCurrency"` // 计算方式是满免的时候会用到
}

type FreightStyle string

const FreightStyleDefault FreightStyle = "default"    // 默认运费计算方式(设置的运费)
const FreightStyleCategory FreightStyle = "category"  // 根据分类计算
const FreightStylePrice FreightStyle = "price"        // 根据页面价格计算
const FreightStyleMinPrice FreightStyle = "min_price" // 根据价格满多少免运费

type MarginConfig struct {
	MarginStyle MarginStyle `json:"marginStyle" bson:"marginStyle"` // 加价方式
	Margin      float64     `json:"margin" bson:"margin"`           // 加价率,加价方式为固定加价时使用
	Ladders     []Ladder    `json:"ladders" bson:"ladders"`         // 阶梯加价,加价方式为阶梯加价时使用
}

func (l Ladder) IsMatch(price float64) bool {
	if l.LastOne && l.Max == 0 {
		return price >= l.Min
	}
	return price >= l.Min && price < l.Max
}

func (mc MarginConfig) IsValidLadders() bool {
	if mc.MarginStyle == MarginStyleLadder {
		if len(mc.Ladders) <= 0 {
			return false
		}
		for i, ladder := range mc.Ladders {
			// 除了最后一个阶梯，其他阶梯的最大值必须设置值
			if i < len(mc.Ladders)-1 && ladder.Max <= 0 {
				return false
			}
			// 阶梯的最小值必须大于等于0
			if ladder.Min < 0 {
				return false
			}
			// 阶梯的最大值必须大于最小值
			if ladder.Min >= ladder.Max {
				return false
			}
			// 阶梯的加价率必须大于0小于等于1
			if ladder.Margin <= 0 || ladder.Margin > 1 {
				return false
			}
		}
	}
	return true
}

type MarginStyle string

const MarginStyleFixed MarginStyle = "fixed"   // 默认加价方式，固定加价一个比例；值为0.1，即加价10%
const MarginStyleLadder MarginStyle = "ladder" // 阶梯加价

type Ladder struct {
	Min     float64 `json:"min" bson:"min"`       // 最小值
	Max     float64 `json:"max" bson:"max"`       // 最大值
	Margin  float64 `json:"margin" bson:"margin"` // 加价率
	LastOne bool    `bson:"-"`
}
