package repository

import (
	"context"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mysql/option"
	"gorm.io/gorm"
)

type BaseInterface interface {
	AutoMigrate() error
}

// Base 基础仓储
type Base[T any] struct {
	db *gorm.DB
}

// NewBase 创建基础仓储
func NewBase[T any](db *gorm.DB) *Base[T] {
	return &Base[T]{db: db}
}

// Create 创建记录
func (receiver *Base[T]) Create(ctx context.Context, entity *T) error {
	return receiver.db.WithContext(ctx).Create(entity).Error
}

// GetByID 根据ID获取记录
func (receiver *Base[T]) GetByID(ctx context.Context, id interface{}) (*T, error) {
	var entity T
	err := receiver.db.WithContext(ctx).First(&entity, id).Error
	if err != nil {
		return nil, err
	}
	return &entity, nil
}

// Update 更新记录
func (receiver *Base[T]) Update(ctx context.Context, entity *T) error {
	return receiver.db.WithContext(ctx).Save(entity).Error
}

// Delete 删除记录
func (receiver *Base[T]) Delete(ctx context.Context, id interface{}) error {
	var entity T
	return receiver.db.WithContext(ctx).Delete(&entity, id).Error
}

// List 查询列表
func (receiver *Base[T]) List(ctx context.Context, opts ...option.Query) ([]*T, error) {
	var entities []*T
	query := receiver.db.WithContext(ctx)

	for _, opt := range opts {
		query = opt(query)
	}

	err := query.Find(&entities).Error
	return entities, err
}

// Count 统计数量
func (receiver *Base[T]) Count(ctx context.Context, opts ...option.Query) (int64, error) {
	var count int64
	query := receiver.db.WithContext(ctx).Model(new(T))

	for _, opt := range opts {
		query = opt(query)
	}

	err := query.Count(&count).Error
	return count, err
}

// Paginate 分页查询
func (receiver *Base[T]) Paginate(ctx context.Context, page, pageSize int, opts ...option.Query) (*PaginateResult[T], error) {
	var entities []*T
	var total int64

	query := receiver.db.WithContext(ctx).Model(new(T))

	// 应用查询条件
	for _, opt := range opts {
		query = opt(query)
	}

	// 获取总数
	if err := query.Count(&total).Error; err != nil {
		return nil, err
	}

	// 分页查询
	offset := (page - 1) * pageSize
	if err := query.Offset(offset).Limit(pageSize).Find(&entities).Error; err != nil {
		return nil, err
	}

	return &PaginateResult[T]{
		Data:     entities,
		Total:    total,
		Page:     page,
		PageSize: pageSize,
		Pages:    int((total + int64(pageSize) - 1) / int64(pageSize)),
	}, nil
}

type PaginateResult[T any] struct {
	Data     []*T
	Total    int64
	Page     int
	PageSize int
	Pages    int
}

func (receiver *Base[T]) AutoMigrate() error {
	var t T
	return receiver.db.AutoMigrate(&t)
}

func (receiver *Base[T]) getTableName(table any) (string, error) {
	tableStmt := &gorm.Statement{DB: receiver.db}
	err := tableStmt.Parse(table)
	if err != nil {
		return "", err
	}
	return tableStmt.Schema.Table, nil
}
