package repository

import (
	"fmt"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mysql/model"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

type CompleteSpuInterface interface {
	BaseInterface
	UpsertCompleteSpus(completeSpus model.CompleteSpus) error
}

type CompleteSpu struct {
	*Base[model.CompleteSpu]
}

func NewCompleteSpu(db *gorm.DB) *CompleteSpu {
	return &CompleteSpu{
		Base: NewBase[model.CompleteSpu](db),
	}
}

func (receiver *CompleteSpu) UpsertCompleteSpus(completeSpus model.CompleteSpus) error {
	return receiver.db.Transaction(func(db *gorm.DB) error {
		return upsertCompleteSpus(db, completeSpus)
	})
}

func upsertCompleteSpus(db *gorm.DB, completeSpus model.CompleteSpus) error {
	// upsert completeSpus
	result := db.Clauses(clause.OnConflict{
		Columns:   []clause.Column{{Name: "style_number"}},
		DoUpdates: clause.AssignmentColumns([]string{"style_number", "title_cn", "title_en", "description", "season", "brand_id", "care", "made_in", "hs_code", "info_select"}),
	}).Create(&completeSpus)
	if result.Error != nil {
		return result.Error
	}

	// OnConflict 导致的更新操作不会回填 ID，即使外键 StyleId 存在，也会报错 WHERE conditions required，所以重新读取出来
	styleNumbers := completeSpus.GetStyleNumbers()
	var reloadedSpus []model.CompleteSpu
	err := db.Where("style_number IN ?", styleNumbers).Find(&reloadedSpus).Error
	if err != nil {
		return fmt.Errorf("failed to reload CompleteSpus: %v", err)
	}
	for _, reloadSpu := range reloadedSpus {
		for i := range completeSpus {
			if reloadSpu.StyleNumber == completeSpus[i].StyleNumber {
				completeSpus[i].ID = reloadSpu.ID
			}
		}
	}

	for _, spu := range completeSpus {
		err := upsertCompleteSpu(db, spu)
		if err != nil {
			return err
		}
	}

	return nil
}

func upsertCompleteSpu(db *gorm.DB, completeSpu model.CompleteSpu) error {
	// 处理关联关系
	// 不能直接 relapce 因为 ColorSpu CompleteSpuStyleNumber not null replace 会先将外键设置为 null 导致报错
	err := db.Model(&completeSpu).Association("ColorSpus").Delete()
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("ColorSpus").Append(completeSpu.ColorSpus)
	if err != nil {
		return err
	}

	styles := completeSpu.ColorSpus.GetStyles()
	var reloadedColorSpus model.ColorSpus
	err = db.Where("(complete_spu_style_number, color_code) IN ?", styles).Find(&reloadedColorSpus).Error
	if err != nil {
		return fmt.Errorf("failed to reload ColorSpus: %v", err)
	}
	for _, reloadColorSpu := range reloadedColorSpus {
		for i := range completeSpu.ColorSpus {
			if completeSpu.ColorSpus[i].CompleteSpuStyleNumber == reloadColorSpu.CompleteSpuStyleNumber &&
				completeSpu.ColorSpus[i].ColorCode == reloadColorSpu.ColorCode {
				completeSpu.ColorSpus[i].ID = reloadColorSpu.ID
			}
		}
	}

	for _, colorSpu := range completeSpu.ColorSpus {
		err = db.Model(&colorSpu).Association("Skus").Delete()
		if err != nil {
			return err
		}
		err = db.Model(&colorSpu).Association("Skus").Append(colorSpu.Skus)
		if err != nil {
			return err
		}
		err = db.Model(&colorSpu).Association("Images").Delete()
		if err != nil {
			return err
		}
		err = db.Model(&colorSpu).Association("Images").Append(colorSpu.Images)
		if err != nil {
			return err
		}
	}
	err = db.Model(&completeSpu).Association("Compositions").Delete()
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Compositions").Append(completeSpu.Compositions)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Models").Delete()
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Models").Append(completeSpu.Models)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Fits").Delete()
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Fits").Append(completeSpu.Fits)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Genders").Delete()
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Genders").Append(completeSpu.Genders)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Categories").Delete()
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Categories").Append(completeSpu.Categories)
	if err != nil {
		return err
	}

	return nil
}
