package convert

import (
	"context"
	"errors"
	"git.chillcy.com/golang/chillcy/pkg/gender"
	"git.chillcy.com/golang/chillcy/pkg/slice"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/pb/pc4"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mongo"
	model "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/product"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/cache"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/rate"
	"go.mongodb.org/mongo-driver/bson"
	mongo2 "go.mongodb.org/mongo-driver/mongo"
	"sync"
)

type Product struct {
	cache          *cache.Cache
	mongoManager   *mongo.Manager
	rate           *rate.Rate
	brandRankCache *cache.BrandRank
}

func NewProduct(cache *cache.Cache, mongoManager *mongo.Manager, rate *rate.Rate, brandRankCache *cache.BrandRank) *Product {
	return &Product{cache: cache, mongoManager: mongoManager, rate: rate, brandRankCache: brandRankCache}
}

func (receiver *Product) Convert(designerProducts []*pc4.DesignerProduct) (model.Products, error) {
	var ps model.Products
	for _, designerProduct := range designerProducts {
		sourceIds := getSourceIds(designerProduct)
		ws := receiver.getWeightSelect(sourceIds)
		once := sync.Once{}
		p := model.Product{}
		styleNumber := designerProduct.GetDesignerId()
		for _, colorProduct := range designerProduct.GetColorProducts() {
			for _, productAndSkus := range colorProduct.GetProductAndSkusList() {
				product := productAndSkus.GetProduct()
				once.Do(func() {
					p.StyleId = styleNumber
					p.StyleNumber = styleNumber
					p.Title = model.ProductTitle{
						En: product.GetTitleEn(),
						Cn: product.GetTitleCn(),
					}
					p.Description = product.GetDetail().GetDescription()
					p.Season = product.GetSeason()
					p.Brand = model.ProductBrand{
						Id:   product.GetBrandId(),
						Name: product.GetBrandName(),
					}
					p.Care = product.GetDetail().GetCare()
					p.MadeIn = product.GetDetail().GetMadeIn()
					p.HsCode = product.GetDetail().GetHsCode()
					p.InfoSelect = product.GetId()
					p.Category = getCategory(product.GetCategoriesV2())
					for _, composition := range product.GetDetail().GetCompositions() {
						p.Compositions = append(p.Compositions, model.ProductComposition{
							Part:     composition.GetPart(),
							Material: composition.GetMaterial(),
							Value:    composition.GetValue(),
						})
					}
					p.Fits = product.GetDetail().GetFits()
					for _, g := range product.GetGenders() {
						p.Genders = append(p.Genders, gender.Gender(g))
					}
					p.Models = product.GetDetail().GetModels()
				})
				if ws.Title.sourceId == product.GetSourceId() {
					p.Title = model.ProductTitle{
						En: product.GetTitleEn(),
						Cn: product.GetTitleCn(),
					}
				}
				if ws.Category.sourceId == product.GetSourceId() {
					p.Category = getCategory(product.GetCategoriesV2())
				}
				if ws.Compositions.sourceId == product.GetSourceId() {
					for _, composition := range product.GetDetail().GetCompositions() {
						p.Compositions = append(p.Compositions, model.ProductComposition{
							Part:     composition.GetPart(),
							Material: composition.GetMaterial(),
							Value:    composition.GetValue(),
						})
					}
				}
				if ws.Care.sourceId == product.GetSourceId() {
					p.Care = product.GetDetail().GetCare()
				}
				if ws.Description.sourceId == product.GetSourceId() {
					p.Description = product.GetDetail().GetDescription()
				}
				if ws.Fits.sourceId == product.GetSourceId() {
					p.Fits = product.GetDetail().GetFits()
				}
				if ws.Models.sourceId == product.GetSourceId() {
					p.Models = product.GetDetail().GetModels()
				}
				if ws.Season.sourceId == product.GetSourceId() {
					p.Season = product.GetSeason()
				}
			}
		}
		var oldProductPointer *model.Product
		oldProduct, err := receiver.mongoManager.ProductModel.FindOne(context.TODO(), bson.M{"styleId": p.StyleId})
		if err != nil && !errors.Is(err, mongo2.ErrNoDocuments) {
			return nil, err
		}
		if !errors.Is(err, mongo2.ErrNoDocuments) {
			oldProductPointer = &oldProduct
		}
		platformStoreSkuConvert := NewPlatformStoreSku(receiver.mongoManager, receiver.rate, receiver.brandRankCache)
		colors, err := buildProductColors(designerProduct.GetColorProducts(), ws, oldProductPointer, platformStoreSkuConvert)
		if err != nil {
			return nil, err
		}
		p.Colors = colors
		ps = append(ps, p)
	}
	return ps, nil
}

func getCategory(category *pc4.CategoryV2) model.ProductCategory {
	pc := model.ProductCategory{}
	pc.Id = category.GetCategoryId()
	pc.Name = category.GetCategoryName()
	if category.GetChildren() == nil {
		return pc
	}
	for _, c := range category.GetChildren() {
		pc.Children = append(pc.Children, getCategory(c))
	}
	return pc
}

func buildProductColors(colorProducts []*pc4.ColorProduct, ws weightSelect, oldProductPointer *model.Product, platformStoreSkuConvert *PlatformStoreSku) (model.ProductColors, error) {
	var pcs model.ProductColors
	for _, colorProduct := range colorProducts {
		pc, err := buildProductColor(colorProduct, ws, oldProductPointer, platformStoreSkuConvert)
		if err != nil {
			return nil, err
		}
		pcs = append(pcs, pc)
	}
	return pcs, nil
}

func buildProductColor(colorProduct *pc4.ColorProduct, ws weightSelect, oldProductPointer *model.Product, platformStoreSkuConvert *PlatformStoreSku) (model.ProductColor, error) {
	pc := model.ProductColor{}
	once := &sync.Once{}
	for _, productAndSkus := range colorProduct.GetProductAndSkusList() {
		product := productAndSkus.GetProduct()
		once.Do(func() {
			pc.ColorId = colorProduct.GetColorNumber()
			pc.ColorCode = colorProduct.GetColorNumber()
			pc.Id = product.GetColor().GetId()
			pc.Name = product.GetColor().GetName()
			pc.BasicId = product.GetColor().GetBasicColorId()
			pc.BasicName = product.GetColor().GetBasicColorName()
			pc.InfoSelect = product.GetId()
			pc.Images = product.GetColor().GetImages()
		})

		if ws.Image.sourceId == product.GetSourceId() {
			pc.Images = product.GetColor().GetImages()
		}

		for _, sku := range productAndSkus.GetSkus() {
			pcSku := model.ProductColorSku{
				Pc3SkuId:     sku.GetId(),
				Pc3ProductId: product.GetId(),
				SourceId:     product.GetSourceId(),
				SiteId:       product.GetSiteId(),
				Size:         sku.GetRawSize(),
				Stock:        int(sku.GetStock()),
				Price:        sku.GetPriceDetail().GetNow(),
				Currency:     sku.GetPriceDetail().GetCurrency(),
				SubjectIds:   product.GetSubjects(),
			}
			psSkus, err := platformStoreSkuConvert.Convert(product, sku, oldProductPointer)
			if err != nil {
				return model.ProductColor{}, err
			}
			pcSku.PlatformStoreSkus = psSkus
			pc.Skus = append(pc.Skus, pcSku)
		}
	}
	return pc, nil
}

func (receiver *Product) getWeightSelect(sourceIds []string) weightSelect {
	sourceItems := receiver.cache.Source.GetSourceItems()
	ws := weightSelect{}
	for _, item := range sourceItems {
		if !slice.Contains(sourceIds, item.Id) { // 不在所需要合并商品中的货源跳过
			continue
		}
		if ws.Image.weight < item.CombineWeight.Image {
			ws.Image.sourceId = item.Id
			ws.Image.weight = item.CombineWeight.Image
		}
		if ws.Title.weight < item.CombineWeight.Title {
			ws.Title.sourceId = item.Id
			ws.Title.weight = item.CombineWeight.Title
		}
		if ws.Category.weight < item.CombineWeight.Category {
			ws.Category.sourceId = item.Id
			ws.Category.weight = item.CombineWeight.Category
		}
		if ws.Compositions.weight < item.CombineWeight.Compositions {
			ws.Compositions.sourceId = item.Id
			ws.Compositions.weight = item.CombineWeight.Compositions
		}
		if ws.Care.weight < item.CombineWeight.Care {
			ws.Care.sourceId = item.Id
			ws.Care.weight = item.CombineWeight.Care
		}
		if ws.Description.weight < item.CombineWeight.Description {
			ws.Description.sourceId = item.Id
			ws.Description.weight = item.CombineWeight.Description
		}
		if ws.Fits.weight < item.CombineWeight.Fits {
			ws.Fits.sourceId = item.Id
			ws.Fits.weight = item.CombineWeight.Fits
		}
		if ws.Models.weight < item.CombineWeight.Models {
			ws.Models.sourceId = item.Id
			ws.Models.weight = item.CombineWeight.Models
		}
		if ws.Season.weight < item.CombineWeight.Season {
			ws.Season.sourceId = item.Id
			ws.Season.weight = item.CombineWeight.Season
		}
	}
	return ws
}
