package convert

import (
	"git.chillcy.com/golang/chillcy/pkg/slice"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/pb/pc4"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mysql/model"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/cache"
	"sync"
)

type Spu struct {
	cache *cache.Cache
}

func NewSpu(cache *cache.Cache) *Spu {
	return &Spu{cache: cache}
}

func (receiver *Spu) Convert(designerProducts []*pc4.DesignerProduct) model.CompleteSpus {
	var completeSpus model.CompleteSpus
	for _, designerProduct := range designerProducts {
		completeSpus = append(completeSpus, receiver.buildCompleteSpu(designerProduct))
	}
	return completeSpus
}

type weight struct {
	sourceId string
	weight   int
}
type weightSelect struct {
	Image        weight
	Title        weight
	Category     weight
	Compositions weight
	Care         weight
	Description  weight
	Fits         weight
	Models       weight
	Season       weight
}

func (receiver *Spu) buildCompleteSpu(designerProduct *pc4.DesignerProduct) model.CompleteSpu {
	sourceIds := getSourceIds(designerProduct)
	ws := receiver.getWeightSelect(sourceIds)
	completeSpu := model.CompleteSpu{}
	completeSpu.StyleNumber = designerProduct.GetDesignerId()
	once := sync.Once{}
	for _, colorProduct := range designerProduct.GetColorProducts() {
		for _, productAndSkus := range colorProduct.GetProductAndSkusList() {
			product := productAndSkus.GetProduct()

			once.Do(func() { // 默认商品数据选择
				completeSpu.InfoSelect = product.GetId()
				completeSpu.Title = model.Title{
					En: product.GetTitleEn(),
					Cn: product.GetTitleCn(),
				}
				completeSpu.Description = product.GetDetail().GetDescription()
				completeSpu.Season = product.GetSeason()
				completeSpu.BrandId = product.GetBrandId()
				completeSpu.Care = product.GetDetail().GetCare()
				completeSpu.MadeIn = product.GetDetail().GetMadeIn()
				completeSpu.HsCode = product.GetDetail().GetHsCode()
				completeSpu.Compositions = buildCompositions(product.GetDetail().GetCompositions(), product.GetDesignerId())
				completeSpu.Models = buildModels(product.GetDetail().GetModels(), product.GetDesignerId())
				completeSpu.Fits = buildFits(product.GetDetail().GetFits(), product.GetDesignerId())
				for _, g := range product.GetGenders() {
					completeSpu.Genders = append(completeSpu.Genders, model.Gender{
						Code: g,
					})
				}
				completeSpu.Categories = buildCategories(product.GetCategoryIds())
			})

			// 分权重商品信息选择覆盖默认

			if product.GetSourceId() == ws.Title.sourceId {
				completeSpu.Title = model.Title{
					En: product.GetTitleEn(),
					Cn: product.GetTitleCn(),
				}
			}

			if product.GetSourceId() == ws.Description.sourceId {
				completeSpu.Description = product.GetDetail().GetDescription()
			}

			if product.GetSourceId() == ws.Season.sourceId {
				completeSpu.Season = product.GetSeason()
			}

			if product.GetSourceId() == ws.Care.sourceId {
				completeSpu.Care = product.GetDetail().GetCare()
			}

			if product.GetSourceId() == ws.Compositions.sourceId {
				completeSpu.Compositions = buildCompositions(product.GetDetail().GetCompositions(), product.GetDesignerId())
			}

			if product.GetSourceId() == ws.Models.sourceId {
				completeSpu.Models = buildModels(product.GetDetail().GetModels(), product.GetDesignerId())
			}

			if product.GetSourceId() == ws.Fits.sourceId {
				completeSpu.Fits = buildFits(product.GetDetail().GetFits(), product.GetDesignerId())
			}

			if product.GetSourceId() == ws.Category.sourceId {
				completeSpu.Categories = buildCategories(product.GetCategoryIds())
			}
		}
	}
	completeSpu.ColorSpus = buildColorSpus(designerProduct.GetColorProducts(), designerProduct.GetDesignerId(), ws)
	return completeSpu
}

func buildCategories(cs []string) model.Categories {
	var categories model.Categories
	for _, c := range cs {
		categories = append(categories, model.Category{
			Code:       c,
			Name:       "",
			ParentCode: "",
		})
	}
	return categories
}

func buildFits(fs []string, styleNumber string) model.Fits {
	var fits model.Fits
	for _, f := range fs {
		fits = append(fits, model.Fit{
			CompleteSpuStyleNumber: styleNumber,
			Content:                f,
		})
	}
	return fits
}

func buildModels(ms []string, styleNumber string) model.Models {
	var models model.Models
	for _, m := range ms {
		models = append(models, model.Model{
			CompleteSpuStyleNumber: styleNumber,
			Content:                m,
		})
	}
	return models
}

func buildCompositions(cs []*pc4.Composition, styleNumber string) model.Compositions {
	var compositions model.Compositions
	for _, c := range cs {
		compositions = append(compositions, model.Composition{
			CompleteSpuStyleNumber: styleNumber,
			Part:                   c.GetPart(),
			Material:               c.GetMaterial(),
			Value:                  c.GetValue(),
		})
	}
	return compositions
}

func buildColorSpus(colorProducts []*pc4.ColorProduct, styleNumber string, ws weightSelect) model.ColorSpus {
	var colorSpus model.ColorSpus
	for _, colorProduct := range colorProducts {
		colorSpu := buildColorSpu(colorProduct, styleNumber, ws)
		colorSpus = append(colorSpus, colorSpu)
	}
	return colorSpus
}

func buildColorSpu(colorProduct *pc4.ColorProduct, styleNumber string, ws weightSelect) model.ColorSpu {
	colorSpu := model.ColorSpu{}
	colorSpu.CompleteSpuStyleNumber = styleNumber
	colorSpu.ColorCode = colorProduct.GetColorNumber()
	once := sync.Once{}
	for _, productAndSkus := range colorProduct.GetProductAndSkusList() {
		product := productAndSkus.GetProduct()
		once.Do(func() {
			colorSpu.InfoSelect = product.GetId()
			colorSpu.Color = model.Color{
				Id:        product.GetColor().GetId(),
				Name:      product.GetColor().GetName(),
				BasicId:   product.GetColor().GetBasicColorId(),
				BasicName: product.GetColor().GetBasicColorName(),
			}

			colorSpu.Images = getImages(product.GetColor().GetImages(), styleNumber, colorProduct.GetColorNumber())
		})

		if product.GetSourceId() == ws.Image.sourceId {
			colorSpu.Images = getImages(product.GetColor().GetImages(), styleNumber, colorProduct.GetColorNumber())
		}
	}
	colorSpu.Skus = buildSkus(colorProduct.GetProductAndSkusList(), styleNumber, colorProduct.GetColorNumber())
	return colorSpu
}

func getImages(imageUrls []string, styleNumber, colorCode string) model.Images {
	var images model.Images
	for _, imageUrl := range imageUrls {
		images = append(images, model.Image{
			ColorSpuStyleNumber: styleNumber,
			ColorSpuColorCode:   colorCode,
			Url:                 imageUrl,
		})
	}
	return images
}

func buildSkus(productAndSkusList []*pc4.ProductAndSkus, styleNumber, colorCode string) model.Skus {
	var skus model.Skus
	for _, productAndSkus := range productAndSkusList {
		for _, pSku := range productAndSkus.GetSkus() {
			sku := model.Sku{}
			sku.Pc3SkuId = pSku.GetId()
			sku.Pc3ProductId = productAndSkus.GetProduct().GetId()
			sku.ColorSpuStyleNumber = styleNumber
			sku.ColorSpuColorCode = colorCode
			sku.SourceId = productAndSkus.GetProduct().GetSourceId()
			sku.SiteId = productAndSkus.GetProduct().GetSiteId()
			sku.Size = pSku.GetRawSize()
			sku.Stock = int(pSku.GetStock())
			sku.Price = pSku.GetPriceDetail().GetNow()
			sku.Currency = pSku.GetPriceDetail().GetCurrency()
			skus = append(skus, sku)
		}
	}
	return skus
}

func getSourceIds(designerProduct *pc4.DesignerProduct) []string {
	var sourceIds []string
	for _, colorProduct := range designerProduct.GetColorProducts() {
		for _, productAndSkus := range colorProduct.GetProductAndSkusList() {
			sourceIds = append(sourceIds, productAndSkus.GetProduct().GetSourceId())
		}
	}
	return sourceIds
}

func (receiver *Spu) getWeightSelect(sourceIds []string) weightSelect {
	sourceItems := receiver.cache.Source.GetSourceItems()
	ws := weightSelect{}
	for _, item := range sourceItems {
		if !slice.Contains(sourceIds, item.Id) { // 不在所需要合并商品中的货源跳过
			continue
		}
		if ws.Image.weight < item.CombineWeight.Image {
			ws.Image.sourceId = item.Id
			ws.Image.weight = item.CombineWeight.Image
		}
		if ws.Title.weight < item.CombineWeight.Title {
			ws.Title.sourceId = item.Id
			ws.Title.weight = item.CombineWeight.Title
		}
		if ws.Category.weight < item.CombineWeight.Category {
			ws.Category.sourceId = item.Id
			ws.Category.weight = item.CombineWeight.Category
		}
		if ws.Compositions.weight < item.CombineWeight.Compositions {
			ws.Compositions.sourceId = item.Id
			ws.Compositions.weight = item.CombineWeight.Compositions
		}
		if ws.Care.weight < item.CombineWeight.Care {
			ws.Care.sourceId = item.Id
			ws.Care.weight = item.CombineWeight.Care
		}
		if ws.Description.weight < item.CombineWeight.Description {
			ws.Description.sourceId = item.Id
			ws.Description.weight = item.CombineWeight.Description
		}
		if ws.Fits.weight < item.CombineWeight.Fits {
			ws.Fits.sourceId = item.Id
			ws.Fits.weight = item.CombineWeight.Fits
		}
		if ws.Models.weight < item.CombineWeight.Models {
			ws.Models.sourceId = item.Id
			ws.Models.weight = item.CombineWeight.Models
		}
		if ws.Season.weight < item.CombineWeight.Season {
			ws.Season.sourceId = item.Id
			ws.Season.weight = item.CombineWeight.Season
		}
	}
	return ws
}
